#!/usr/bin/python
import sys
import getopt
import os
import os.path
import signal
import diskless 
import gettext
import gtk
import gnome
import gobject
import gtk.glade
import shutil
import commands
import pxeboot
from pxeos import *
from pxeosdialog import *
from rhpl import *
import Alchemist
from netboot_util import *
from netboot_gtk import *

#Columns for "treeview"
COLUMN_OS=0
COLUMN_DESCRIPTION=1
COLUMN_DATA=2

quiting=0

gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)

try:
    gettext.install(PROGNAME, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode    

def idle_func():
    while gtk.events_pending():
        gtk.mainiteration()

class FirstTime:
    """This class handles everything gui for the redhat-config-netboot application"""
    def __init__(self,xml,parent):
        self.xml = xml
        self.needsNFS=FALSE
        self.needsFTP=FALSE
        self.needsHTTPD=FALSE

        self.firsttime = self.xml.get_widget("netbootFirsttime")
        self.initDruid = self.xml.get_widget("initDruid")
        self.initDruid.connect("cancel",self.cancel)
        self.pxeosDialog=pxeosdialog(self.xml)

        self.xml.get_widget("druidpagefinish1").connect("finish",self.on_initDruid_finish)
        self.pxeoslist=pxeOSList()
        # map the event signals to specific methods
        self.xml.get_widget("remoteInstallPage").connect("next",self.on_remoteInstallPage_next)
        self.xml.get_widget("addOSButton").connect("clicked",self.on_addOSButton_clicked)
        self.xml.get_widget("deleteOSButton").connect("clicked",self.on_deleteOSButton_clicked)
        self.xml.get_widget("propertiesOSButton").connect("clicked",self.on_propertiesOSButton_clicked)
        self.setupList()
        self.serverName=hostname()
        self.setDHCPText()
        
    def setDHCPText(self):
        output= """
allow booting;
allow bootp;

class "pxeclients" {
        match if substring (option vendor-class-identifier, 0, 9) = "PXEClient";
        filename "/linux-install/pxelinux.0";
	next-server %s
}""" %  self.serverName
        self.xml.get_widget("dhcpTextview").get_buffer().set_text(output)
        
        
    def get_runlevel(self):
        """returns the current runlevel, uses /sbin/runlevel"""
        runlevel_output = commands.getoutput("/sbin/runlevel")
        # This is the current runlevel
        return int(runlevel_output[-1])

    def isServiceRunning(self,service,runlevel=-1):
        if runlevel == -1:
            runlevel=self.get_runlevel()
        rc=commands.getoutput("/sbin/chkconfig --list %s" % service).split()
        if len(rc) == 2:
            return rc[1]=="on"
        else:
            return rc[runlevel+1].split(":")[-1]=="on"

    def quit(self,arg1=None,arg2=None):
        global quiting
        quiting=1
        gtk.mainquit()

    def __setServiceMessage(self,required,protocol):
        label=("%sLabel"%protocol)
        if not self.isServiceRunning(protocol):
            if required:
                self.xml.get_widget(label).set_markup ('<span color="red">%s</span>' % _("The %s service is required for network booting and will be configured to run." % protocol))
            else:
                self.xml.get_widget(label).set_text (_("The %s service is running, but it is not required for network booting." % protocol))
        else:
            self.xml.get_widget(label).set_text (_("The %s service is not being used and is not required for network booting." % protocol))

        
    def on_remoteInstallPage_next(self,*args):
        model = self.tree_view.get_model ()
        if model.iter_n_children(None)==0:
            errorMessage(_("You must enter at least one Operating System"))
            return TRUE
        protocol=[0,0,0]
        def foreach_func (model, path, iter, protocol):
            OS = model.get_value (iter, COLUMN_DATA)
            p=OS.getProtocol()
            if p=="NFS":
                self.needsNFS=TRUE
            if p=="FTP":
                self.needsFTP=TRUE
            if p=="HTTP":
                self.needsHTTPD=TRUE
            return FALSE
        model.foreach (foreach_func, protocol)

        if not self.isServiceRunning("tftp"):
            self.xml.get_widget("tftpLabel").set_markup ('<span color="red">%s</span>' % _("The tftp service will be configured to run."))

        self.__setServiceMessage(self.needsFTP,"ftp")
        self.__setServiceMessage(self.needsNFS,"nfs")
        self.__setServiceMessage(self.needsHTTPD,"httpd")
        
    def run(self):
        self.firsttime.show_all()
        rc=self.firsttime.run()
        self.firsttime.hide()
        return rc

    def hide(self):
        self.firsttime.hide()

    def cancel(self,button):
        self.firsttime.response(gtk.RESPONSE_CANCEL)

    def on_initDruid_finish(self,*args):
        localip=self.serverName
        model = self.tree_view.get_model ()
        osList=[]
        def foreach_func (model, path, iter, osList):
            OS = model.get_value (iter, COLUMN_DATA)
            osList.append(OS)
            self.setupPXEdir(OS,localip)
            return FALSE
        model.foreach (foreach_func, osList)

        mkdir("%s/pxelinux.cfg" % PXEBOOTDIR)
        os.system ("/usr/bin/rsync -ap %s/msgs %s" % (INSTALLDIR,PXEBOOTDIR))
        os.system ("/usr/bin/rsync -ap %s/pxelinux.cfg %s" % (INSTALLDIR,PXEBOOTDIR))

        bootFD=open("%s/msgs/boot.msg" % PXEBOOTDIR,"a")
        defaultFD=open("%s/pxelinux.cfg/default" % PXEBOOTDIR,"a")
        self.__startServices()

        self.pxeoslist.saveOSList(osList)
        self.firsttime.response(gtk.RESPONSE_APPLY)

    def __startServices(self):
        if self.needsNFS:
            os.system("/sbin/chkconfig nfs on")
            if not self.isServiceRunning("nfs"):
                os.system("/sbin/service nfs start")

        if self.needsFTP:
            os.system("/sbin/chkconfig ftp on")
            if not self.isServiceRunning("ftp"):
                os.system("/sbin/service ftp start")

        if self.needsHTTPD:
            os.system("/sbin/chkconfig httpd on")
            if not self.isServiceRunning("httpd"):
                os.system("/sbin/service httpd start")

        if not self.isServiceRunning("tftp"):
            os.system("/sbin/chkconfig tftp on")
            os.system("/sbin/service xinetd reload")

    def setupPXEdir(self,OS,host):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE,
                                _("Setting up installation directory for %s" % OS.getName()))
        dlg.show_all()
        dlg.show_now()
        idle_func()
        rhdir="%s/%s" % (PXEBOOTDIR,OS.getNameDir())
        protocol=translateProtocol(OS.getProtocol())
        location = OS.getLocation()

        idle_func()
        filename = "%s/ks.cfg" % (rhdir)
        if not OS.isDiskless():
            ks=open(filename,"w")
            ks.write("#use %s installation media\n" % protocol)
            if protocol=="NFS":
                ks.write("nfs --server %s --dir %s\n" % (host,location))
            if protocol=="HTTP":
                ks.write("url --url http://%s/%s\n" % (host,location))
            if protocol=="FTP":
                if OS.getUser()!="":
                    ks.write("url --url ftp://%s:%s@%s/%s\n" % (host,OS.getUser(),OS.getPassword(),location))
                else:
                    ks.write("url --url ftp://%s/%s\n" % (host,location))
            ks.close()
        dlg.destroy()

    def on_deleteOSButton_clicked(self,*args):
        """Callback for the deleteButton."""
        result = self.tree_view.get_selection().get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                os = model.get_value (iter, COLUMN_DATA)
            except TypeError:
                return
            message=_("Are you sure you want to delete the '%s' Operating System config?") % os.getName()
            dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                    gtk.BUTTONS_YES_NO,
                                    message)
            dlg.show_all()
            rc = dlg.run()
            dlg.destroy()
            if rc==gtk.RESPONSE_YES:
                model.remove(iter)

    def on_addOSButton_clicked(self,*args):
        self.on_edit()

    def on_edit(self,OS=None,iter=None):
        model=self.tree_view.get_model ()
        if OS == None:
            OS=pxeos()
        rc=self.pxeosDialog.run(OS)
        if rc==gtk.RESPONSE_OK:
            print OS.getName(),OS.getDescription()
            if iter==None:
                iter = model.append ()
            model.set (iter,
                       COLUMN_OS,OS.getName(),
                       COLUMN_DESCRIPTION,OS.getDescription(),
                       COLUMN_DATA, OS)
                
    
    def on_propertiesOSButton_clicked(self,*args):
        """Callback for the propertiesButton."""
        result = self.tree_view.get_selection().get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                os = model.get_value (iter, COLUMN_DATA)
            except TypeError:
                return
            self.on_edit(os,iter)

    def setupList(self):
        """Populates osTreeview with the os files"""
        self.tree_view = self.xml.get_widget("osTreeview")
        model = gtk.ListStore (gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)
        model.set_sort_column_id(1, gtk.SORT_ASCENDING)        
        self.tree_view.set_model (model)
        column = gtk.TreeViewColumn (_("OS Name"), gtk.CellRendererText(), text=COLUMN_OS)
        column.set_sort_column_id(1)
        self.tree_view.append_column (column)
        column = gtk.TreeViewColumn (_("Description"), gtk.CellRendererText(), text=COLUMN_DESCRIPTION)
        column.set_sort_column_id(2)
        self.tree_view.append_column (column)
        self.tree_view.connect ("row_activated",self.on_tree_view_row_activated,
                               self.on_propertiesOSButton_clicked)

        for OS in self.pxeoslist.getOSList():
            if OS.isDiskless():
                continue
            iter = model.append ()
            model.set (iter,
                       COLUMN_OS,OS.getName(),
                       COLUMN_DESCRIPTION,OS.getDescription(),
                       COLUMN_DATA, OS)

        selection=self.tree_view.get_selection()
        selection.select_path ((0,))

    def on_tree_view_row_activated (self,tree_view, path, column, func):
        apply (func)

if __name__ == "__main__":
    cmdline = sys.argv[1:]
    sys.argv = sys.argv[:1]

