#!/usr/bin/python
## Copyright (C) 2003 Red Hat, Inc.
## Copyright (C) 2003 Dan Walsh <dwalsh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import gettext
from netboot_util import *
from pxeos import *
from rhpl.translate import _, N_, cat

def hex2ip(hex):
    """ hex2ip will throw an exception on a badly formed ip"""
    ret=""
    for i in xrange((len(hex)+1)/2):
        hexno = '0x'+hex[2*i:2*i+2]
        if ret != "":
            ret = ret + "."
        ret = ret+ ("%d" % eval(hexno))
    return ret

def getPXEFiles():
    l={}
    if os.path.isdir(PXECFGDIR):
        for i in os.listdir(PXECFGDIR):
            try:
                l[i]=hex2ip(i)
            except:
                pass
    return l

class pxeboot:
    def __init__(self,val):
        try:
            s=socket.inet_aton(val)
            ip=val
        except socket.error,message:
            ip=socket.gethostbyname(val)
        self.ip=ip
        self.path = PXECFGDIR + self.getHEXIP().strip()
        self.setKickstart("")
        self.setSnapshot("")
        self.append=""
        self.setRamdisk("10000")
        self.setEthernet("eth0")
        self.setSerial(FALSE)
        self.os=None
        self.dict={}
        try:
            self.load()
        except:
            pass
        
    def __getNFSRoot(self,pxeos):
        return "root=/dev/ram0 init=disklessrc NFSROOT=%s:%s" % (pxeos.getServer(),pxeos.getLocation())

    def getIP(self):
        return self.ip;
    
    def getHostname(self):
        try:
            return socket.gethostbyaddr(self.ip)[0];
        except:
            return self.ip
    
    def getEthernet(self):
        return self.ethernet

    def setEthernet(self,ethernet):
        self.ethernet=ethernet
    
    def __getEthernetString(self):
        return "ETHERNET="+self.getEthernet()

    def isSerial(self):
        return self.serial

    def setSerial(self,serial):
        self.serial=serial
    
    def __getConsoleString(self):
        if self.isSerial():
            return "console=ttyS0"
        else:
            return ""

    def setRamdisk(self,ramdisk):
        self.ramdisk=int(ramdisk)

    def getRamdisk(self):
        return self.ramdisk

    def __getRamdiskString(self):
        if self.getRamdisk()==0:
            return ""
        else:
            return "ramdisk_size="+str(self.getRamdisk())

    def setKickstart(self,kickstart):
        self.kickstart=kickstart

    def getKickstart(self):
        return self.kickstart
    def __getKickstartString(self):
        return "ks=%s" % self.getKickstart()

    def setSnapshot(self,snapshot):
        self.snapshot=snapshot

    def getSnapshot(self):
        if self.snapshot=="":
            return self.getHostname()
        else:
            return self.snapshot

    def __getSnapshotString(self):
        if self.snapshot=="":
            return ""
        else:
            return "SNAPSHOT="+self.snapshot

    def setOS(self,OS):
        if OS==None:
            raise ValueError,_("You must specify an Operating System")
        l=pxeOSList().getOSList()
        for o in l:
            if o.getName() == OS:
                self.os=OS
                return
        raise ValueError,_("%s is not PXE Operating System") % OS
        
    def getOS(self):
        return self.os
        
    def getpxeos(self):
        l=pxeOSList().getOSList()
        for o in l:
            if o.getName() == self.os:
                return o
        raise ValueError,_("%s is not PXE Operating System") % self.os
        
    def getHEXIP(self):
        out=""
        for i in str.split(self.ip,"."):
            out=out + ("%02x" % int(i))
        return str.upper(out)
    
    def getFilePath(self):
        return self.path

    def isDiskless(self):
        return self.getpxeos().isDiskless()
        
    def __getLabel(self):
        OS=pxeos()
        OS.setName(self.os)
        return OS.getNameDir()

    def out(self):
        pxeos=self.getpxeos()
        label=pxeos.getNameDir()
        return ("default %s\n\nlabel %s\n    kernel %s/vmlinuz\n%s" % (label,label,label, self.__getAppend(pxeos)))

    def __getAppend(self,pxeos):
        appendLine=""
        if pxeos.isDiskless():
            appendLine= "    append %s initrd=%s/initrd.img %s %s %s %s" % (self.__getConsoleString(), self.__getLabel(), self.__getNFSRoot(pxeos),self.__getRamdiskString(),self.__getEthernetString(),self.__getSnapshotString())
        else:
            appendLine = "    append initrd=%s/initrd.img %s %s %s " % (self.__getLabel(), self.__getConsoleString(),self.__getKickstartString(),self.__getRamdiskString())
        for var in self.dict.keys():
            appendLine = "%s %s=%s" % (appendLine,var,self.dict[var])

        return appendLine
        
    def save(self):
        f = open(self.path,"w")
        f.write(self.out())
        f.close()
        pxeos= self.getpxeos()
        if pxeos.isDiskless():
            getstatusoutput("%s/diskless/syncfiles %s %s %s " % (INSTALLDIR, pxeos.getLocation(), self.getSnapshot(), pxeos.getKernel()))
        
    def handleAppend(self,append):
        for i in append:
            fields=i.split("=")
            if fields[0]=="ks":
                self.setKickstart(fields[1])
                continue
            if fields[0]=="SNAPSHOT":
                self.setSnapshot(fields[1])
                continue
            if fields[0]=="ramdisk_size":
                self.setRamdisk(int(fields[1]))
                continue
            if fields[0]=="ETHERNET":
                self.setEthernet(fields[1])
                continue
            if fields[0]=="console":
                self.setSerial(fields[1]=="ttyS0")
                continue
            if fields[0]=="initrd":
                continue
            self.dict[fields[0]]=fields[1]

    def load(self):
        f = open(self.path,"r")
        lines = f.readlines()
        for i in lines:
            fields=i.split()
            if len(fields) > 0:
                if fields[0]=="label":
                    k=fields[1].split("_")
                    OS=""
                    for s in k:
                        OS=OS+s+" "
                    self.setOS(OS.strip())
                    
                if fields[0]=="append":
                    self.handleAppend(fields[1:])
        f.close()
        self.getpxeos()
        
    def delete(self):
        pxeos= self.getpxeos()
        os.system("rm -rf %s/snapshot/%s" % (pxeos.getLocation(), self.getIP()))
        os.remove(self.path)

def Usage():
    print _("pxeboot - Python pxeboot commandline tool\n\nUsage: pxeboot [-a|-m|-d|-l] [-h] [ -K <kickstart> ] [-S <snapshotname> ] [-r <ramdisk>] [-s] [-e <ethernet>] [-O <os>] hostname | IPAddress")

if __name__ == "__main__":
    import sys
    import signal
    import getopt
    cmdline = sys.argv[1:]
    sys.argv = sys.argv[:1]

    if os.getuid() != 0:
        print _("Please restart %s with root permissions!") % (sys.argv[0])
        sys.exit(10)
        
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    class BadUsage: pass
    if len(cmdline)==0:
        Usage()
        sys.exit(10)
        
    ip = cmdline[-1]
    kickstart=None
    snapshot=None
    ramdisk=None
    ethernet="eth0"
    serial=FALSE
    OS=None
    add=FALSE
    modify=FALSE
    delete=FALSE
    progname = os.path.basename(sys.argv[0])

    try:
        opts, args = getopt.getopt(cmdline, "asmde:K:r:O:lh", ["add","modify","delete", "kickstart=","snapshot=","ramdisk=","serial", "ethernet=","os=","list","help"])
        for opt, val in opts:
            
            if opt == '-a' or opt == '--add':
                add=TRUE

            if opt == '-m' or opt == '--modify':
                modify=TRUE

            if opt == '-d' or opt == '--delete':
                delete=TRUE

            if opt == '-K' or opt == '--kickstart':
                kickstart=val

            if opt == '-S' or opt == '--snapshot':
                snapshot=val

            if opt == '-r' or opt == '--ramdisk':
                ramdisk=val

            if opt == '-e' or opt == '--ethernet':
                ethernet=val

            if opt == '-s' or opt == '--serial':
                serial=TRUE

            if opt == '-O' or opt == '--os':
                OS=val

            if opt == '-l' or opt == '--list':
                for i in getPXEFiles().values():
                    pxe=pxeboot(i)
                    try:
                        print pxe.out()
                    except ValueError,message:
                        print _("\nError:Invalid PXE Boot File %s%s\n%s\n" % (PXECFGDIR,i,message))
                        continue
                        
                sys.exit(0)

            if opt == '-h' or opt == '--help':
                Usage()
                sys.exit(0)

    except (getopt.error, BadUsage):
        Usage()
        sys.exit(1)

    if (add and delete) or (add and modify) or (modify and delete):
        Usage()
        sys.exit(1)

    try:
        if delete:
            pxe=pxeboot(ip)
            pxe.delete()
            sys.exit(0)

        if add or modify:
            if add:
                for i in getPXEFiles().values():
                    if i==ip:
                        print "%s already exists" % ip
                        sys.exit(0)
                    
                pxe=pxeboot(ip)
        
            if modify:
                found=FALSE
                for i in getPXEFiles().values():
                    if i==ip:
                        found=TRUE
                        break
                if not found:
                    print "%s does not exist" % ip
                    sys.exit(0)
                
            pxe=pxeboot(ip)
            if kickstart != None:
                pxe.setKickstart(kickstart)
            if snapshot != None:
                pxe.setSnapshot(snapshot)
            if ramdisk != None:
                pxe.setRamdisk(ramdisk)
            if ethernet != None:
                pxe.setEthernet(ethernet)
            pxe.setSerial(serial)
            if OS != None:
                pxe.setOS(OS)
            pxe.save()
            sys.exit(0)
            
        found=FALSE
        for i in getPXEFiles().values():
            if i==ip:
                found=TRUE
                break
        if not found:
            print "%s does not exist" % ip
            sys.exit(0)
        
        pxe=pxeboot(ip)
        pxe.out()
        
    except ValueError,e:
        print e
        sys.exit(1)
        
